####################################################################################
# In this script, data is simulated according to the full response mixture model. 
# Next, the full response mixture model and the baseline model are fit to the data
# ##################################################################################

library(MASS)
library(R2OpenBUGS)
set.seed()

N=2000
nit=20

output.full=matrix(,200,4125)
output.vdl=matrix(,200,4083)

for(iter in 1:80){
theta=matrix(rnorm(N))                        # latent ability                                         
tau=.4*theta+sqrt(1-.4^2)*rnorm(N)            # latent speed, cor(abil,speed)=.4
tau=tau*.4                                    # multiply with standard deviation (sd_speed=0.4)

item.par=read.table("full.itempar",he=F)      # read item parameters
 
a.fast=exp(item.par[,1])                      #alpha_0
a.slow=exp(item.par[,2])                      #alpha_1
b.fast=item.par[,3]                           #beta_0
b.slow=item.par[,4]                           #beta_1
ti=item.par[,5]                               #lambda_i
tr=exp(item.par[,6])                          #sigma_epsilon^2

X=lnT=matrix(,N,nit)                          # X responses and log response time

ti=matrix(ti,N,nit,T)
tr=diag(tr)


b.slow=matrix(b.slow,N,nit,T)
b.fast=matrix(b.fast,N,nit,T)

a.fast=matrix(a.fast,nit,1)
a.slow=matrix(a.slow,nit,1)

err=mvrnorm(N,rep(0,nit),tr)                  #residual response time
lrt=ti - tau%x%matrix(1,1,nit) + err          #log response time model

marg=.5                                       # zeta_0
time=1                                        # zeta_1

pi=plogis(time/sqrt(tr[1,1])*(err)-marg)      #pi_{pi}

dev.slow=theta%x%t(a.slow) - b.slow           
dev.fast=theta%x%t(a.fast) - b.fast

p = pi*plogis(dev.slow) + (1-pi)*plogis(dev.fast)  # probability correct

data=((p>(matrix(runif(N*nit),N,nit)))*1)          # simulate response data

data->x

################################################################
##### FIT RESP MIX USING R Package R2OpenBUGS and OpenBUGS #####
################################################################

item.par=cbind(                                     # matrix of item parameters
c(rep(0,nit)),
c(rep(0,nit)),
qlogis(apply(x,2,mean,na.rm=T)),
qlogis(apply(x,2,mean,na.rm=T)),
rep(1,nit),
rep(0,nit))

item.mu=apply(item.par,2,mean)                     #item parameter means
item.prec=diag(rep(.1,6))                          #item parameter precision

inits1=list(list(item.par=item.par,theta=rnorm(N),speed=rnorm(N),cor=0,prec.s=1,lntime=0,marg=0,item.mu=item.mu,item.prec=item.prec))   #draw initial values

dat=list(x=data,lrt=lrt,N=N,nit=nit,zeros=rep(0,6),R=diag(rep(0.01,6)))                                                               # specify data

nms=c("item.par","cor","prec.s","time","marg","speed","theta")                                                                        # parameters to track during sampling

res=bugs(dat, inits1, nms, n.iter=8002, model.file="full.txt",                                                                        # call OpenBUGS
    n.chains=1, n.burnin=4002, n.thin=1,                                                                                              # the model file (full.txt) is also part of the supplementary material
    debug=FALSE, DIC=TRUE, digits=5)

output.full[iter,1:4125]=colMeans(res$sims.matrix)                                                                                    #parameter means

########################
#####    FIT VDL   #####
########################
rm(res)

item.par=cbind(                                                                                                               # similar as the above
c(rep(0,nit)),
qlogis(apply(x,2,mean,na.rm=T)),
rep(1,nit),
rep(0,nit))

item.mu=apply(item.par,2,mean)
item.prec=diag(rep(.1,4))

inits1=list(list(item.par=item.par,theta=rnorm(N),speed=rnorm(N),cor=0,prec.s=1,item.mu=item.mu,item.prec=item.prec))

dat=list(x=data,lrt=lrt,N=N,nit=nit,zeros=rep(0,4),R=diag(rep(0.01,4)))

nms=c("item.par","cor","prec.s","speed","theta")


res=bugs(dat, inits1, nms, n.iter=8002, model.file="baseline.txt",                                                            # the model file (baseline.txt) is also part of the supplementary material
    n.chains=1, n.burnin=4002, n.thin=1, 
    debug=FALSE, DIC=TRUE, digits=5)

output.vdl[iter,1:4083]=colMeans(res$sims.matrix)

}   
    


     


